import guidance

def test_geneach():
    """ Test a geneach loop.
    """

    llm = guidance.llms.Mock({
        'Joe</item>': {"text": '</list>', "finish_reason": "stop"},
        '</item>': {"text": '\n<item', "finish_reason": "length"},
        '">' : ["Bob", "Sue", "Joe"],
    })
    prompt = guidance('''<instructions>Generate a list of three names</instructions>
<list>{{#geneach 'names' stop="</list>"}}
<item index="{{@index}}">{{gen 'this'}}</item>{{/geneach}}</list>''', llm=llm)
    out = prompt()
    assert len(out["names"]) == 3
    assert out["names"] == ["Bob", "Sue", "Joe"]
    assert str(out) == """<instructions>Generate a list of three names</instructions>
<list>
<item index="0">Bob</item>
<item index="1">Sue</item>
<item index="2">Joe</item></list>"""

def test_geneach_with_join():
    """ Test a geneach loop.
    """

    llm = guidance.llms.Mock({
        'Joe</item>': {"text": '</list>', "finish_reason": "stop"},
        '</item>': {"text": '\n<item', "finish_reason": "length"},
        '">' : ["Bob", "Sue", "Joe"],
    })
    prompt = guidance('''<instructions>Generate a list of three names</instructions>
<list>{{#geneach 'names' join="<mark>" stop="</list>"}}
<item index="{{@index}}">{{gen 'this'}}</item>{{/geneach}}</list>''', llm=llm)
    out = prompt()
    assert len(out["names"]) == 3
    assert out["names"] == ["Bob", "Sue", "Joe"]
    assert str(out) == """<instructions>Generate a list of three names</instructions>
<list>
<item index="0">Bob</item><mark>
<item index="1">Sue</item><mark>
<item index="2">Joe</item></list>"""

def test_geneach_single_call():
    """ Test a geneach loop.
    """

    llm = guidance.llms.Mock('''
<item index="0">Bob</item>
<item index="1">Sue</item>
<item index="2">Jow</item>
</list>''')
    prompt = guidance('''<instructions>Generate a list of three names</instructions>
<list>{{#geneach 'names' single_call=True stop="</list>"}}
<item index="{{@index}}">{{gen 'this'}}</item>{{/geneach}}</list>"''', llm=llm)
    out = prompt()
    assert len(out["names"]) == 3
